@@ -136,9 +136,9 @@ pub trait Vecf64 {
     fn lintrans(self) -> Vec<f64>;
     /// Sorted vector
     fn sortf(self) -> Vec<f64>;
-    /// Sorted vector, is wrapper for mergesort below
+    /// Sorted vector, sortf is wrapper for mergesort below
     fn sortm(self) -> Vec<f64>;
-    /// Ranking in n*log(n), using 'mergesort'
+    /// Ranking with only n*log(n) complexity, using 'mergesort'
     fn mergerank(self) -> Vec<usize>;
     /// Immutable merge sort, makes a sort index
     fn mergesort(self, i:usize, n:usize) -> Vec<usize>;
@@ -213,7 +213,12 @@ pub trait VecVec {
     fn acentroid(self) -> Vec<f64>;
     /// Harmonic Centroid = harmonic mean of a set of points
     fn hcentroid(self) -> Vec<f64>; 
+    /// Trend between two sets
+    fn trend(self, eps: f64, v: Vec<Vec<f64>>) -> Vec<f64>;
+    /// Subtract m from all points - e.g. transform to zero median form
+    fn translate(self, m: &[f64]) -> Vec<Vec<f64>>;
     /// Sums of distances from each point to all other points
+ 
     fn distsums(self) -> Vec<f64>;
     /// Sum of distances from one point given by indx
     fn distsuminset(self, indx: usize) -> f64;
@@ -226,36 +231,27 @@ pub trait VecVec {
  
     /// Eccentricity vectors from each point
     fn eccentricities(self) -> Vec<Vec<f64>>;
-    /// Ecentricity scalar measure of an internal point given by indx
-    fn eccentrinset(self, indx: usize) -> f64;
+    /// Ecentricity of a member point given by its indx
+    fn eccmember(self, indx: usize) -> Vec<f64>;
     /// Eccentricity vector for a non member point
-    fn eccnonmember(self, p:&[f64]) -> Vec<f64>;
-    /// Eccentricity scalar measure and vector of any point     
-    fn veccentr(self, thisp: &[f64]) -> (f64, Vec<f64>);
-    /// Eccentricity scalar measure only, of any point
-    fn ecc(self, v: &[f64]) -> f64;
+    fn eccnonmember(self, p:&[f64]) -> Vec<f64>; 
     /// magnitudes of a set of vectors
-    fn mags(self) -> Vec<f64>;
-    /// scaled magnitudes (typically of eccentricities measures)
-    fn scalarecc(self) -> Vec<f64>;
+    fn mags(self) -> Vec<f64>; 
     /// Median and quartiles of eccentricities (new robust measure of spread of a multivariate sample)
     fn moe(self) -> (MStats,Med);
     /// Medoid and Outlier as defined by eccentricities.
     fn emedoid(self) -> (f64, usize, f64, usize);
 
-    /// Geometric median of a set
-    fn nmedian(self, eps: f64) -> Vec<f64>;
-    /// Betterpoint gives new approximation to nmedian
-    fn betterpoint(self, v: &[f64]) -> (f64, Vec<f64>);
+    /// Geometric medians of a set
+
     /// First iteration point for geometric medians
     fn firstpoint(self) -> Vec<f64>;
-    /// Trend between two sets
-    fn trend(self, eps: f64, v: Vec<Vec<f64>>) -> Vec<f64>;
-    /// Subtract m from all points - e.g. transform to zero median form
-    fn translate(self, m: &[f64]) -> Vec<Vec<f64>>;
-    /// Vector iterative two point method for finding the geometric median
+
+    /// Improved Weizsfeld's Algorithm for geometric median
+    fn nmedian(self, eps: f64) -> Vec<f64>;
+    /// Vector iterative two point method for geometric median
     fn gmedian(self, eps: f64) -> Vec<f64>;
-    /// Secant nethod for finding the geometric median
+    /// Secant nethod for geometric median
     fn smedian(self, eps: f64) -> Vec<f64>;
  
 }
